@@ -6,6 +6,7 @@ import asyncio
 from collections.abc import Callable
 import os
 import ssl
+from urllib.parse import urlparse
 
 import aiofiles
 from cryptography import x509
@@ -18,6 +19,14 @@ from .remote import RemoteProtocol
 from .remotemessage_pb2 import RemoteDirection
 
 
+def _load_cert_chain(certfile: str, keyfile: str) -> ssl.SSLContext:
+    ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+    ssl_context.check_hostname = False
+    ssl_context.verify_mode = ssl.VerifyMode.CERT_NONE
+    ssl_context.load_cert_chain(certfile, keyfile)
+    return ssl_context
+
+
 class AndroidTVRemote:
     """Pairing and connecting to an Android TV for remotely sending commands to it."""
 
@@ -52,7 +61,7 @@ class AndroidTVRemote:
         self._pair_port = pair_port
         self._loop = loop or asyncio.get_running_loop()
         self._enable_ime = enable_ime
-        self._transport = None
+        self._transport: asyncio.Transport | None = None
         self._remote_message_protocol: RemoteProtocol | None = None
         self._pairing_message_protocol: PairingProtocol | None = None
         self._reconnect_task: asyncio.Task | None = None
@@ -62,19 +71,19 @@ class AndroidTVRemote:
         self._volume_info_updated_callbacks: list[Callable] = []
         self._is_available_updated_callbacks: list[Callable] = []
 
-        def is_on_updated(is_on: bool):
+        def is_on_updated(is_on: bool) -> None:
             for callback in self._is_on_updated_callbacks:
                 callback(is_on)
 
-        def current_app_updated(current_app: str):
+        def current_app_updated(current_app: str) -> None:
             for callback in self._current_app_updated_callbacks:
                 callback(current_app)
 
-        def volume_info_updated(volume_info: dict[str, str | bool]):
+        def volume_info_updated(volume_info: dict[str, str | bool]) -> None:
             for callback in self._volume_info_updated_callbacks:
                 callback(volume_info)
 
-        def is_available_updated(is_available: bool):
+        def is_available_updated(is_available: bool) -> None:
             for callback in self._is_available_updated_callbacks:
                 callback(is_available)
 
@@ -105,7 +114,7 @@ class AndroidTVRemote:
         return self._remote_message_protocol.device_info
 
     @property
-    def volume_info(self) -> dict[str, str | bool] | None:
+    def volume_info(self) -> dict[str, str | bool | int] | None:
         """Volume info (level, max, muted)."""
         if not self._remote_message_protocol:
             return None
@@ -169,20 +178,27 @@ class AndroidTVRemote:
             await out.write(key_pem.decode("utf-8"))
         return True
 
-    async def async_connect(self):
+    async def _create_ssl_context(self) -> ssl.SSLContext:
+        if self._ssl_context:
+            return self._ssl_context
+        try:
+            ssl_context = await self._loop.run_in_executor(
+                None, _load_cert_chain, self._certfile, self._keyfile
+            )
+        except FileNotFoundError as exc:
+            LOGGER.debug("Missing certificate. Error: %s", exc)
+            raise InvalidAuth from exc
+        self._ssl_context = ssl_context
+        return self._ssl_context
+
+    async def async_connect(self) -> None:
         """Connect to an Android TV.
 
         :raises CannotConnect: if couldn't connect, e.g. invalid IP address.
         :raises ConnectionClosed: if connection was lost while waiting for the remote to start.
         :raises InvalidAuth: if pairing is needed first.
         """
-        if self._ssl_context is None:
-            self._ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS)
-            try:
-                self._ssl_context.load_cert_chain(self._certfile, self._keyfile)
-            except FileNotFoundError as exc:
-                LOGGER.debug("Missing certificate. Error: %s", exc)
-                raise InvalidAuth from exc
+        ssl_context = await self._create_ssl_context()
         on_con_lost = self._loop.create_future()
         on_remote_started = self._loop.create_future()
         try:
@@ -201,12 +217,14 @@ class AndroidTVRemote:
                 ),
                 self.host,
                 self._api_port,
-                ssl=self._ssl_context,
+                ssl=ssl_context,
             )
         except OSError as exc:
             LOGGER.debug(
                 "Couldn't connect to %s:%s. Error: %s", self.host, self._api_port, exc
             )
+            if isinstance(exc, ssl.SSLError):
+                raise InvalidAuth("Need to pair") from exc
             raise CannotConnect(
                 f"Couldn't connect to {self.host}:{self._api_port}"
             ) from exc
@@ -243,7 +261,7 @@ class AndroidTVRemote:
                     await self.async_connect()
                     self._on_is_available_updated(True)
                     break
-                except CannotConnect as exc:
+                except (CannotConnect, ConnectionClosed) as exc:
                     delay_seconds = min(2 * delay_seconds, 30)
                     LOGGER.debug(
                         "Couldn't reconnect to %s. Will retry in %s seconds. Error: %s",
@@ -285,14 +303,14 @@ class AndroidTVRemote:
 
         :raises CannotConnect: if couldn't connect, e.g. invalid IP address.
         """
-        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS)
+        ssl_context = await self._create_ssl_context()
         try:
             _, writer = await asyncio.open_connection(
-                self.host, self._api_port, ssl=ssl_context
+                self.host, self._pair_port, ssl=ssl_context
             )
         except OSError as exc:
             LOGGER.debug(
-                "Couldn't connect to %s:%s. %s", self.host, self._api_port, exc
+                "Couldn't connect to %s:%s. %s", self.host, self._pair_port, exc
             )
             raise CannotConnect from exc
         server_cert_bytes = writer.transport.get_extra_info("ssl_object").getpeercert(
@@ -300,24 +318,28 @@ class AndroidTVRemote:
         )
         writer.close()
         server_cert = x509.load_der_x509_certificate(server_cert_bytes)
-        server_cert_common_name = str(
-            server_cert.subject.get_attributes_for_oid(x509.oid.NameOID.COMMON_NAME)[
-                0
-            ].value
-        )
-        # Example: atvremote/darcy/darcy/SHIELD Android TV/XX:XX:XX:XX:XX:XX
-        parts = server_cert_common_name.split("/")
-        return parts[-2], parts[-1]
+        # NVIDIA SHIELD example:
+        # CN=atvremote/darcy/darcy/SHIELD Android TV/XX:XX:XX:XX:XX:XX
+        # Nexus Player example:
+        # dnQualifier=fugu/fugu/Nexus Player/CN=atvremote/XX:XX:XX:XX:XX:XX
+        common_name = server_cert.subject.get_attributes_for_oid(x509.OID_COMMON_NAME)
+        common_name_str = str(common_name[0].value) if common_name else ""
+        dn_qualifier = server_cert.subject.get_attributes_for_oid(x509.OID_DN_QUALIFIER)
+        dn_qualifier_str = str(dn_qualifier[0].value) if dn_qualifier else ""
+        common_name_parts = common_name_str.split("/")
+        dn_qualifier_parts = dn_qualifier_str.split("/")
+        name = dn_qualifier_parts[-1] if dn_qualifier_str else common_name_parts[-2]
+        mac = common_name_parts[-1]
+        return name, mac
 
-    async def async_start_pairing(self):
+    async def async_start_pairing(self) -> None:
         """Start the pairing process.
 
         :raises CannotConnect: if couldn't connect, e.g. invalid IP address.
         :raises ConnectionClosed: if connection was lost.
         """
         self.disconnect()
-        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS)
-        ssl_context.load_cert_chain(self._certfile, self._keyfile)
+        ssl_context = await self._create_ssl_context()
         on_con_lost = self._loop.create_future()
         try:
             (
@@ -341,7 +363,7 @@ class AndroidTVRemote:
             raise CannotConnect from exc
         await self._pairing_message_protocol.async_start_pairing()
 
-    async def async_finish_pairing(self, pairing_code: str):
+    async def async_finish_pairing(self, pairing_code: str) -> None:
         """Finish the pairing process.
 
         :param pairing_code: pairing code shown on the Android TV.
@@ -356,7 +378,7 @@ class AndroidTVRemote:
 
     def send_key_command(
         self, key_code: int | str, direction: int | str = RemoteDirection.SHORT
-    ):
+    ) -> None:
         """Send a key press to Android TV.
 
         This does not block; it buffers the data and arranges for it to be sent out asynchronously.
@@ -372,7 +394,19 @@ class AndroidTVRemote:
             raise ConnectionClosed("Called send_key_command after disconnect")
         self._remote_message_protocol.send_key_command(key_code, direction)
 
-    def send_launch_app_command(self, app_link: str):
+    def send_text(self, text: str) -> None:
+        """Send text to Android TV.
+
+        :param text: text to be sent.
+        :raises ConnectionClosed: if client is disconnected.
+        :may not work as expected if virtual keyboard is present on the Android TV screen
+        """
+        if not self._remote_message_protocol:
+            LOGGER.debug("Called send_text after disconnect")
+            raise ConnectionClosed("Called send_text after disconnect")
+        self._remote_message_protocol.send_text(text)
+
+    def send_launch_app_command(self, app_link_or_app_id: str) -> None:
         """Launch an app on Android TV.
 
         This does not block; it buffers the data and arranges for it to be sent out asynchronously.
@@ -382,4 +416,7 @@ class AndroidTVRemote:
         if not self._remote_message_protocol:
             LOGGER.debug("Called send_launch_app_command after disconnect")
             raise ConnectionClosed("Called send_launch_app_command after disconnect")
-        self._remote_message_protocol.send_launch_app_command(app_link)
+        prefix = "" if urlparse(app_link_or_app_id).scheme else "market://launch?id="
+        self._remote_message_protocol.send_launch_app_command(
+            f"{prefix}{app_link_or_app_id}"
+        )
