@@ -15,11 +15,20 @@ from google.protobuf.message import DecodeError
 
 from .base import ProtobufProtocol
 from .const import LOGGER
-from .remotemessage_pb2 import RemoteDirection, RemoteKeyCode, RemoteMessage
+from .remotemessage_pb2 import (
+    RemoteDirection,
+    RemoteEditInfo,
+    RemoteImeBatchEdit,
+    RemoteImeObject,
+    RemoteKeyCode,
+    RemoteMessage,
+)
 
 LOG_PING_REQUESTS = False
 ERROR_SUGGESTION_MSG = (
-    "Try clearing the storage of the Android TV Remove Service system app."
+    "Try clearing the storage of the Android TV Remote Service system app. "
+    "On the Android TV device, go to Settings > Apps > See all apps > Show system apps. "
+    "Then, select Android TV Remote Service > Storage > Clear data/storage."
 )
 
 
@@ -37,7 +46,7 @@ class Feature(IntFlag):
 class RemoteProtocol(ProtobufProtocol):
     """Implement remote protocol with an Android TV.
 
-    Messages transmitted between client and server are of type RemoteMessage, see remotegmessage.proto.
+    Messages transmitted between client and server are of type RemoteMessage, see remotemessage.proto.
     Protocol is described in
     https://github.com/Aymkdn/assistant-freebox-cloud/wiki/Google-TV-(aka-Android-TV)-Remote-Control-(v2)
     """
@@ -79,14 +88,16 @@ class RemoteProtocol(ProtobufProtocol):
         self.is_on = False
         self.current_app = ""
         self.device_info: dict[str, str] = {}
-        self.volume_info: dict[str, str | bool] = {}
+        self.volume_info: dict[str, str | bool | int] = {}
+        self.ime_counter: int = 0
+        self.ime_field_counter: int = 0
         self._loop = loop
         self._idle_disconnect_task: asyncio.Task | None = None
         self._reset_idle_disconnect_task()
 
     def send_key_command(
         self, key_code: int | str, direction: int | str = RemoteDirection.SHORT
-    ):
+    ) -> None:
         """Send a key press to Android TV.
 
         This does not block; it buffers the data and arranges for it to be sent out asynchronously.
@@ -104,11 +115,36 @@ class RemoteProtocol(ProtobufProtocol):
             key_code = RemoteKeyCode.Value(key_code)
         if isinstance(direction, str):
             direction = RemoteDirection.Value(direction)
-        msg.remote_key_inject.key_code = key_code
-        msg.remote_key_inject.direction = direction
+        msg.remote_key_inject.key_code = key_code  # type: ignore[assignment]
+        msg.remote_key_inject.direction = direction  # type: ignore[assignment]
         self._send_message(msg)
 
-    def send_launch_app_command(self, app_link: str):
+    def send_text(self, text: str) -> None:
+        """Send a text string to Android TV via the input method.
+
+        The text length is used for both `start` and `end` in the RemoteImeObject.
+        The `ime_counter` and `ime_field_counter` values are taken from self (batch_edit_info response),
+        which is populated when a message with a remote_ime_batch_edit field is received.
+
+        :param text: The text string to be sent.
+        """
+        if not text:
+            raise ValueError("Text cannot be empty")
+
+        self._reset_idle_disconnect_task()
+        msg = RemoteMessage()
+        param_value = len(text) - 1
+        ime_object = RemoteImeObject(start=param_value, end=param_value, value=text)
+        edit_info = RemoteEditInfo(insert=1, text_field_status=ime_object)
+        batch_edit = RemoteImeBatchEdit(
+            ime_counter=self.ime_counter,
+            field_counter=self.ime_field_counter,
+            edit_info=[edit_info],
+        )
+        msg.remote_ime_batch_edit.CopyFrom(batch_edit)
+        self._send_message(msg)
+
+    def send_launch_app_command(self, app_link: str) -> None:
         """Launch an app on Android TV.
 
         This does not block; it buffers the data and arranges for it to be sent out asynchronously.
@@ -118,7 +154,7 @@ class RemoteProtocol(ProtobufProtocol):
         msg.remote_app_link_launch_request.app_link = app_link
         self._send_message(msg)
 
-    def _handle_message(self, raw_msg):
+    def _handle_message(self, raw_msg: bytes) -> None:
         """Handle a message from the server."""
         self._reset_idle_disconnect_task()
         msg = RemoteMessage()
@@ -163,6 +199,9 @@ class RemoteProtocol(ProtobufProtocol):
         elif msg.HasField("remote_ime_key_inject"):
             self.current_app = msg.remote_ime_key_inject.app_info.app_package
             self._on_current_app_updated(self.current_app)
+        elif msg.HasField("remote_ime_batch_edit"):
+            self.ime_counter = msg.remote_ime_batch_edit.ime_counter
+            self.ime_field_counter = msg.remote_ime_batch_edit.field_counter
         elif msg.HasField("remote_set_volume_level"):
             self.volume_info = {
                 "level": msg.remote_set_volume_level.volume_level,
@@ -186,14 +225,14 @@ class RemoteProtocol(ProtobufProtocol):
         if new_msg != RemoteMessage():
             self._send_message(new_msg, log_send)
 
-    def _reset_idle_disconnect_task(self):
+    def _reset_idle_disconnect_task(self) -> None:
         if self._idle_disconnect_task is not None:
             self._idle_disconnect_task.cancel()
         self._idle_disconnect_task = self._loop.create_task(
             self._async_idle_disconnect()
         )
 
-    async def _async_idle_disconnect(self):
+    async def _async_idle_disconnect(self) -> None:
         # Disconnect if there is no message from the server or client within
         # 16 seconds. Server pings every 5 seconds if there is no command sent.
         # This is similar to the server behavior that closes connections after 3
