@@ -19,7 +19,7 @@ from androidtvremote2 import (
 _LOGGER = logging.getLogger(__name__)
 
 
-async def _bind_keyboard(remote: AndroidTVRemote):
+async def _bind_keyboard(remote: AndroidTVRemote) -> None:
     print(
         "\n\nYou can control the connected Android TV with:"
         "\n- arrow keys: move selected item"
@@ -34,7 +34,9 @@ async def _bind_keyboard(remote: AndroidTVRemote):
         "\n- 'n': Netflix"
         "\n- 'd': Disney+"
         "\n- 'a': Amazon Prime Video"
-        "\n- 'q': quit\n\n"
+        "\n- 'k': Kodi"
+        "\n- 'q': quit"
+        "\n- 't': send text 'Hello world' to Android TV\n\n"
     )
     key_mappings = {
         keyboard.Key.up: "DPAD_UP",
@@ -49,11 +51,11 @@ async def _bind_keyboard(remote: AndroidTVRemote):
         keyboard.Key.delete: "POWER",
     }
 
-    def transmit_keys():
-        queue = asyncio.Queue()
+    def transmit_keys() -> asyncio.Queue:
+        queue: asyncio.Queue = asyncio.Queue()
         loop = asyncio.get_event_loop()
 
-        def on_press(key):
+        def on_press(key: keyboard.Key | keyboard.KeyCode | None) -> None:
             loop.call_soon_threadsafe(queue.put_nowait, key)
 
         keyboard.Listener(on_press=on_press).start()
@@ -77,11 +79,15 @@ async def _bind_keyboard(remote: AndroidTVRemote):
             elif key.char == "y":
                 remote.send_launch_app_command("https://www.youtube.com")
             elif key.char == "n":
-                remote.send_launch_app_command("https://www.netflix.com/title")
+                remote.send_launch_app_command("com.netflix.ninja")
             elif key.char == "d":
-                remote.send_launch_app_command("https://www.disneyplus.com")
+                remote.send_launch_app_command("com.disney.disneyplus")
             elif key.char == "a":
-                remote.send_launch_app_command("https://app.primevideo.com")
+                remote.send_launch_app_command("com.amazon.amazonvideo.livingroom")
+            elif key.char == "k":
+                remote.send_launch_app_command("org.xbmc.kodi")
+            elif key.char == "t":
+                remote.send_text("Hello World!")
 
 
 async def _host_from_zeroconf(timeout: float) -> str:
@@ -93,7 +99,7 @@ async def _host_from_zeroconf(timeout: float) -> str:
     ) -> None:
         if state_change is not ServiceStateChange.Added:
             return
-        _ = asyncio.ensure_future(
+        _ = asyncio.ensure_future(  # noqa: RUF006
             async_display_service_info(zeroconf, service_type, name)
         )
 
@@ -104,38 +110,38 @@ async def _host_from_zeroconf(timeout: float) -> str:
         await info.async_request(zeroconf, 3000)
         if info:
             addresses = [
-                "%s:%d" % (addr, cast(int, info.port))
+                f"{addr}:{cast(int, info.port)}"
                 for addr in info.parsed_scoped_addresses()
             ]
-            print("  Name: %s" % name)
-            print("  Addresses: %s" % ", ".join(addresses))
+            print(f"  Name: {name}")
+            print(f"  Addresses: {", ".join(addresses)}")
             if info.properties:
                 print("  Properties:")
                 for key, value in info.properties.items():
-                    print(f"    {key}: {value}")
+                    print(f"    {key!r}: {value!r}")
             else:
                 print("  No properties")
         else:
             print("  No info")
         print()
 
-    aiozc = AsyncZeroconf()
+    zc = AsyncZeroconf()
     services = ["_androidtvremote2._tcp.local."]
     print(
         f"\nBrowsing {services} service(s) for {timeout} seconds, press Ctrl-C to exit...\n"
     )
-    aiobrowser = AsyncServiceBrowser(
-        aiozc.zeroconf, services, handlers=[_async_on_service_state_change]
+    browser = AsyncServiceBrowser(
+        zc.zeroconf, services, handlers=[_async_on_service_state_change]
     )
     await asyncio.sleep(timeout)
 
-    await aiobrowser.async_cancel()
-    await aiozc.async_close()
+    await browser.async_cancel()
+    await zc.async_close()
 
     return input("Enter IP address of Android TV to connect to: ").split(":")[0]
 
 
-async def _pair(remote: AndroidTVRemote):
+async def _pair(remote: AndroidTVRemote) -> None:
     name, mac = await remote.async_get_name_and_mac()
     if (
         input(
@@ -158,7 +164,7 @@ async def _pair(remote: AndroidTVRemote):
             return await _pair(remote)
 
 
-async def _main():
+async def _main() -> None:
     parser = argparse.ArgumentParser()
     parser.add_argument("--host", help="IP address of Android TV to connect to")
     parser.add_argument(
@@ -215,16 +221,16 @@ async def _main():
     _LOGGER.info("current_app: %s", remote.current_app)
     _LOGGER.info("volume_info: %s", remote.volume_info)
 
-    def is_on_updated(is_on):
+    def is_on_updated(is_on: bool) -> None:
         _LOGGER.info("Notified that is_on: %s", is_on)
 
-    def current_app_updated(current_app):
+    def current_app_updated(current_app: str) -> None:
         _LOGGER.info("Notified that current_app: %s", current_app)
 
-    def volume_info_updated(volume_info):
+    def volume_info_updated(volume_info: dict[str, str | bool]) -> None:
         _LOGGER.info("Notified that volume_info: %s", volume_info)
 
-    def is_available_updated(is_available):
+    def is_available_updated(is_available: bool) -> None:
         _LOGGER.info("Notified that is_available: %s", is_available)
 
     remote.add_is_on_updated_callback(is_on_updated)
